@@ -1,14 +1,14 @@
-# Copyright 2002 by Roger Bivand and Michael Tiefelsdorf
+# Copyright 2002-2008 by Roger Bivand and Michael Tiefelsdorf
 #
 
 lm.morantest.sad <- function (model, listw, zero.policy = FALSE, 
     alternative = "greater", spChk=NULL, resfun=weighted.residuals, 
     tol = .Machine$double.eps^0.5, maxiter = 1000, tol.bounds=0.0001,
-    zero.tol=1.0e-7) 
+    zero.tol=1.0e-7, Omega=NULL, save.M=NULL, save.U=NULL) 
 {
     if (!inherits(listw, "listw")) 
         stop(paste(deparse(substitute(listw)), "is not a listw object"))
-    if (class(model) != "lm") 
+    if (!inherits(model, "lm")) 
         stop(paste(deparse(substitute(model)), "not an lm object"))
     N <- length(listw$neighbours)
     u <- resfun(model)
@@ -40,17 +40,24 @@ lm.morantest.sad <- function (model, listw, zero.policy = FALSE,
     }
     M <- diag(N) - X %*% XtXinv %*% t(X)
     U <- listw2mat(listw.U)
-    MVM <- M %*% U %*% M
-    MVM <- 0.5 * (t(MVM) + MVM)
-    evalue <- eigen(MVM, only.values=TRUE)$values
-    idxpos <- which(abs(evalue) < zero.tol)
-    if (length(idxpos) != p)
-        warning("number of zero eigenvalues greater than number of variables")
-    idxpos <- idxpos[1] - 1
-    if (idxpos < 1) stop("invalid first zero eigenvalue index")
-    tau <- evalue[1:idxpos]
-    tau <- c(tau, evalue[(idxpos+1+p):N])
-    mres <- moranSad(tau, I, tol, maxiter, tol.bounds, alternative=alternative)
+    if (is.null(Omega)) {
+        MVM <- M %*% U %*% M
+        MVM <- 0.5 * (t(MVM) + MVM)
+        evalue <- eigen(MVM, only.values=TRUE)$values
+        idxpos <- which(abs(evalue) < zero.tol)
+        if (length(idxpos) != p)
+            warning("number of zero eigenvalues greater than number of variables")
+        idxpos <- idxpos[1] - 1
+        if (idxpos < 1) stop("invalid first zero eigenvalue index")
+        tau <- evalue[1:idxpos]
+        tau <- c(tau, evalue[(idxpos+1+p):N])
+        mres <- moranSad(tau, I, tol, maxiter, tol.bounds,
+            alternative=alternative, type="Global")
+    } else {
+        if (dim(Omega)[1] != N) stop("Omega of different size than data")
+        mres <- moranSadAlt(I, M, U, Omega, N, tol=tol, maxiter=maxiter,
+            tol.bounds=tol.bounds, alternative=alternative)
+    }
     statistic <- mres$sad.p
     attr(statistic, "names") <- "Saddlepoint approximation"
     p.value <- mres$p.sad
@@ -71,30 +78,49 @@ lm.morantest.sad <- function (model, listw, zero.policy = FALSE,
 	internal1 = internal1, internal2 = internal2,
 	df = (N-p), tau = tau)
     class(res) <- "moransad"
+    if (!is.null(save.M)) res$M <- M
+    if (!is.null(save.U)) res$U <- U
     return(res)
 }
 
+moranSadAlt <- function(I, M, U, Omega, n, tol=.Machine$double.eps^0.5,
+    maxiter=1000, tol.bounds=0.0001, alternative="greater") {
+    Omega <- chol(Omega)
+    A <- Omega %*% M %*% (U- diag(I, n)) %*% M %*% t(Omega)
+    tau <- sort(eigen(A)$values, decreasing=TRUE)
+    obj <- moranSad(tau, I, tol=tol, maxiter=maxiter,
+            tol.bounds=tol.bounds, alternative=alternative, type="Alternative")
+    obj
+}
+
 moranSad <- function(tau, I, tol=.Machine$double.eps^0.5, maxiter=1000,
-    tol.bounds=0.0001, alternative="greater") {
-    taumi <- tau - I
+    tol.bounds=0.0001, alternative="greater", type="Global") {
+    if (type == "Global") taumi <- tau - I
+    else if (type == "Alternative") taumi <- tau
     low <- (1 / (2*taumi[length(taumi)])) + tol.bounds
     high <- (1 / (2*taumi[1])) - tol.bounds
-    f <- function(omega, taumi) {sum(taumi/(1 - (2*omega*taumi)))}
-    root <- uniroot(f, lower=low, upper=high, tol=tol, maxiter=maxiter,
-        taumi=taumi)
-    omega <- root$root
-    if (omega < 0 ) sad.r <- -sqrt(sum(log(1 - 2*omega*taumi)))
-    else sad.r <- sqrt(sum(log(1 - 2*omega*taumi)))
-    sad.u <- omega * sqrt(2*sum(taumi^2 / (1 - (2*omega*taumi))^2))
-    sad.p <- sad.r - ((1/sad.r)*log(sad.r/sad.u))
-    if (alternative == "two.sided") p.sad <- 2 * pnorm(abs(sad.p), 
-	lower.tail=FALSE)
-    else if (alternative == "greater")
-        p.sad <- pnorm(sad.p, lower.tail=FALSE)
-    else p.sad <- pnorm(sad.p)
-    if (p.sad < 0 || p.sad > 1) 
-	warning("Out-of-range p-value: reconsider test arguments")
-    return(list(p.sad=p.sad, sad.p=sad.p, sad.r=sad.r, sad.u=sad.u, omega=omega, root=root))
+    if (!(low < high)) {
+        omega <- root <- p.sad <- sad.p <- sad.r <- sad.u <- NA
+        warning("low not less than high for uniroot in moranSad")
+    } else {
+        f <- function(omega, taumi) {sum(taumi/(1 - (2*omega*taumi)))}
+        root <- uniroot(f, lower=low, upper=high, tol=tol, maxiter=maxiter,
+            taumi=taumi)
+        omega <- root$root
+        if (omega < 0 ) sad.r <- -sqrt(sum(log(1 - 2*omega*taumi)))
+        else sad.r <- sqrt(sum(log(1 - 2*omega*taumi)))
+        sad.u <- omega * sqrt(2*sum(taumi^2 / (1 - (2*omega*taumi))^2))
+        sad.p <- sad.r - ((1/sad.r)*log(sad.r/sad.u))
+        if (alternative == "two.sided") p.sad <- 2 * pnorm(abs(sad.p), 
+	    lower.tail=FALSE)
+        else if (alternative == "greater")
+            p.sad <- pnorm(sad.p, lower.tail=FALSE)
+        else p.sad <- pnorm(sad.p)
+        if (!is.finite(p.sad) || p.sad < 0 || p.sad > 1) 
+	    warning("Out-of-range p-value: reconsider test arguments")
+    }
+    return(list(p.sad=p.sad, sad.p=sad.p, sad.r=sad.r, sad.u=sad.u,
+        omega=omega, root=root))
 }
 
 print.moransad <- function(x, ...) {
